// Copyright 2021-2022 @skyekiwi/s-contract authors & contributors
// SPDX-License-Identifier: Apache-2.0
import { ApiPromise, WsProvider } from '@polkadot/api';
import { mnemonicToMiniSecret, mnemonicValidate } from '@polkadot/util-crypto';
import { baseDecode, baseEncode } from 'borsh';
import { DefaultSealer, EncryptionSchema } from '@skyekiwi/crypto';
import { Driver } from '@skyekiwi/driver';
import { hexToU8a } from '@skyekiwi/util';
import { buildCalls, parseCalls } from "./borsh.js";
export class ContractController {
  static buildInitialState(calls) {
    return baseDecode(buildCalls(calls));
  }

  static parseInitialState(encodedCalls) {
    return parseCalls(baseEncode(encodedCalls));
  }

  static intoSecretContract(calls, wasmBlob) {
    return {
      initialState: ContractController.buildInitialState(calls),
      wasmBlob: wasmBlob
    };
  }

  static async upstream(registry, contract) {
    const mnemonic = process.env.SEED_PHRASE;

    if (!mnemonicValidate(mnemonic)) {
      throw new Error('mnemonic failed to read - e2e.spec.ts');
    }

    if (contract.initialState.length !== 0) {
      const provider = new WsProvider('wss://staging.rpc.skye.kiwi');
      const api = await ApiPromise.create({
        provider: provider
      });
      const encryptionSchema = new EncryptionSchema();
      const sksRaw = await api.query.registry.secretKeepers();
      const sks = sksRaw.toJSON();
      const sealer = new DefaultSealer();
      sealer.unlock(mnemonicToMiniSecret(process.env.SEED_PHRASE));

      if (!sks || sks.length === 0) {
        console.warn('no secret keepers found. Testing mode only.');
        encryptionSchema.addMember(sealer.getAuthorKey());
      } else {
        for (const sk of sks) {
          const pk = (await api.query.registry.publicKey(sk)).toString();
          encryptionSchema.addMember(hexToU8a(pk));
        }
      }

      await provider.disconnect();
      return await Driver.upstreamContract(registry, contract, sealer, encryptionSchema);
    } else {
      return await Driver.upstreamContract(registry, contract);
    }
  }

  static async downstream(registry, secretId) {
    const mnemonic = process.env.SEED_PHRASE;

    if (!mnemonicValidate(mnemonic)) {
      throw new Error('mnemonic failed to read - e2e.spec.ts');
    }

    const sealer = new DefaultSealer();
    sealer.unlock(mnemonicToMiniSecret(mnemonic));
    return await Driver.downstreamContract(secretId, registry, [mnemonicToMiniSecret(mnemonic)], sealer);
  }

}