// Copyright 2021-2022 @skyekiwi/s-contract authors & contributors
// SPDX-License-Identifier: Apache-2.0
import { buildBlock, buildCall, buildContract, buildExecutionSummary, buildLocalMetadata, buildOutcome, buildShard, buildShardMetadata, parseBlock, parseCall, parseContract, parseExecutionSummary, parseLocalMetadata, parseOutcome, parseShard, parseShardMetadata } from "./borsh.js";

const numberPadding = (n, pad) => {
  return String(n).padStart(pad, '0');
};
/* eslint-disable sort-keys, camelcase, @typescript-eslint/ban-ts-comment */


export class Storage {
  static getCallIndex(shardId, callIndex) {
    const shard = numberPadding(shardId, 4);
    const block = numberPadding(callIndex, 16);
    return shard + block + 'RAWC';
  }

  static getCallOutcomeIndex(shardId, callIndex) {
    const shard = numberPadding(shardId, 4);
    const block = numberPadding(callIndex, 16);
    return shard + block + 'OUTC';
  }

  static getBlockIndex(shardId, blockNumber) {
    const shard = numberPadding(shardId, 4);
    const block = numberPadding(blockNumber, 16);
    return shard + block + 'BLOC';
  }

  static getContractIndex(contractIndex) {
    return numberPadding(contractIndex, 20) + 'CONT';
  }

  static getShardIndex(shardId) {
    return numberPadding(shardId, 20) + 'SHAR';
  }

  static getShardMetadataIndex(shardId) {
    return numberPadding(shardId, 20) + 'SHAM';
  }

  static writeMetadata(metadata) {
    return {
      type: 'put',
      key: 'METADATA',
      value: buildLocalMetadata(metadata)
    };
  }

  static async getMetadata(db) {
    return parseLocalMetadata(await db.get('METADATA'));
  }

  static writeCallRecord(shard_id, callIndex, call) {
    const key = Storage.getCallIndex(shard_id, callIndex);
    return {
      type: 'put',
      key: key,
      value: buildCall(call)
    };
  }

  static writeCallOutcome(shardId, callIndex, outcome) {
    const key = Storage.getCallOutcomeIndex(shardId, callIndex);
    return {
      type: 'put',
      key: key,
      value: buildOutcome(outcome)
    };
  }

  static writeBlockRecord(shardId, blockNumber, block) {
    const key = Storage.getBlockIndex(shardId, blockNumber);
    return {
      type: 'put',
      key: key,
      value: buildBlock(block)
    };
  }

  static writeContractRecord(contractIndex, contract) {
    const key = Storage.getContractIndex(contractIndex);
    return {
      type: 'put',
      key: key,
      value: buildContract(contract)
    };
  }

  static writeShardRecord(shardId, shard) {
    const key = Storage.getShardIndex(shardId);
    return {
      type: 'put',
      key: key,
      value: buildShard(shard)
    };
  }

  static writeShardMetadataRecord(shardId, shardm) {
    const key = Storage.getShardIndex(shardId);
    return {
      type: 'put',
      key: key,
      value: buildShardMetadata(shardm)
    };
  }

  static writeExecutionSummary(a) {
    return {
      type: 'put',
      key: 'EXECUTION_SUMMARY',
      value: buildExecutionSummary(a)
    };
  }

  static async writeAll(db, ops) {
    // eslint-diable
    // @ts-ignore
    await db.batch(ops); // eslint-enable
  }

  static async getCallRecord(db, shardId, callIndex) {
    const key = Storage.getCallIndex(shardId, callIndex);
    return parseCall(await db.get(key));
  }

  static async getOutcomeRecord(db, shardId, callIndex) {
    const key = Storage.getCallOutcomeIndex(shardId, callIndex);
    return parseOutcome(await db.get(key));
  }

  static async getBlockRecord(db, shardId, blockNumber) {
    const key = Storage.getBlockIndex(shardId, blockNumber);
    return parseBlock(await db.get(key));
  }

  static async getContractRecord(db, contractIndex) {
    const key = Storage.getContractIndex(contractIndex);
    return parseContract(await db.get(key));
  }

  static async getShardRecord(db, shardId) {
    const key = Storage.getShardIndex(shardId);
    return parseShard(await db.get(key));
  }

  static async getShardMetadataRecord(db, shardId) {
    const key = Storage.getShardMetadataIndex(shardId);
    return parseShardMetadata(await db.get(key));
  }

  static async getExecutionSummary(db) {
    return parseExecutionSummary(await db.get('EXECUTION_SUMMARY'));
  }

}